function  res=Propose3(action,a1,a2)
%   PROPOSE - PROPagation Of SEcond harmonic wave in medical ultrasound imaging
%
%   PROPOSE('1inear',z2):       Linear wave propagation to z-position z2
%   PROPOSE('non1inear',z2):    NonLinear wave propagation of 2.harmonic wave to zposition z2
%   PROPOSE('both',z2):         is equivalent to PROPOSE('1inear',z2);PROPOSE('non1inear',z2);
%   PROPOSE('2way',z2,{[dx,dy]})   calculate 2-way response in z=z2, with receiver skew dx,dy [m]
%   val=PROPOSE('get',parameter) read out parameter values or simulation data
%   Execute PROPOSE('get') to list all available parameter and data names 
%   PROPOSE('setparam',parameter,value) Set parameter values
%   Aperture shape: Ashapetx,Ashaperx 'rectangular' or 'elliptic_exp', (superelliptic with exp>=2)
%   PROPOSE('setdata',parameter,value)  Modify simulation data 
%   PROPOSE('plot',{option},{figNo})             Plot simulation results
%   PROPOSE('clear')            clear all simulation data (but not parameter values)
%   PROPOSE('init')             initialize PROPOSE and set default values

% Hans Torp 31.05.02
% Correction for different azimuth/elevation ROC  02.06.02  HT
% Calculation of 2way response included     03.06.02  HT
% superelliptic shape of aperture            04.06.02  HT
% bugfix and comments for parameters and data included    14.08.02  HT
% minor changes 14.11.02  HT
% changed from kaiser to Tukey window for apodisation
% corrected circsym. case  10.07.3  HT
% Corrected 2way steering Torbjrn Hergum 18.7.03

% Fungerer n ogs i nrfelt. rettet endel feil i FieldFoc og focalcorrection 3.05.2011 HT
% ROCcorrection for different azimuth and elevation focus also works! Set
% ROCcorrection=1 in FieldFoc
% Calibration of 2.harm level included 2011.05.07 HT
% Minor changes in prop2harm for paralell looping (for -> parfor)
% Slower on my computer....   2011.05.18 Hans
  
  
global pSettings pSettingsDoc pData pDataDoc;
if isempty(pSettings), [pSettings,pSettingsDoc]=defaultsettings;end;
if nargin<3,a2=[];end;
if nargin<2,a1=[];end;
if nargin<1,action='both';a1=pSettings.Rtx(1);end;%linear and nonlinear prop. to focal plane

res=[];
switch lower(action),   
 case {'linear'},    res=reCalc('propLin',a1);
 case {'nonlinear'}, res=reCalc('prop2harm',a1);
 case {'both'}, 
  reCalc('propLin',a1);
  res=reCalc('prop2harm',a1);
 case {'2way'},    
  res=reCalc('2wayResponse',a1,a2);
 case {'get'},
  if isempty(a1), disp('PropoSe settings:');disp(pSettings);disp(pSettingsDoc);
    disp('PropoSe data:');disp(pData);disp(pDataDoc);
  elseif isfield(pSettings,a1), res=getfield(pSettings,a1);
  elseif isfield(pData,a1), res=getfield(pData,a1);
  end;
 case {'setparam'},
  if isempty(a1),disp('PropoSe settings:');disp(pSettings);
  else res=setfield(pSettings,a1,a2);pSettings=res;end;
 case {'setdata'},setfield(pData,a1,a2);
 case {'plot'},plotResults(a1,a2);
 case {'clear'}, pData=[];
 case {'init'}, pData=[];[pSettings,pSettingsDoc]=defaultsettings; 
 otherwise, reCalc(lower(action),a1,a2);
  %disp('Unknown command in Propose');
end;%switch lower(action)

%%%%%%%%%%%%%%%%%%    end of Propose   %%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [s,sdoc]=defaultsettings

s.c=1498;sdoc.c='speedOfSound';
s.useSymmetry=0;sdoc.useSymmetry='Boolean: "1" allows prop2harm.cpp to use symmetry in calculaton of the 2. order harmonic field. "0" forbids this.';
%transmit aperture
s.Dtx=[20,12]*1e-3;sdoc.Dtx='Transmit aperture diameter azimuth, elevation';
s.Ashapetx='rectangular';sdoc.Ashapetx='Aperture shape transmit =rectangular or elliptic_exp for superelliptic form';
s.Rtx=[60,60]*1e-3;sdoc.Rtx='Transmit aperture focal distance azimuth, elevation';
s.Apodtx=[0,0];sdoc.Apodtx='Transmit apodization azimuth, elevation (R values for Tukey window)';
s.f0tx=2.0e6;sdoc.f0tx='centerfrequency transmitted pulse';
s.Btx=0.3e6;sdoc.Btx='pulse bandwidth transmitted pulse';
%receiver aperture
s.Drx=[20,12]*1e-3;sdoc.Drx='Receive aperture diameter azimuth, elevation';
s.Ashaperx='rectangular';sdoc.Ashaperx='Aperture shape receive';
s.Rrx=[60,60]*1e-3;sdoc.Rrx='Receiver aperture focal distance azimuth, elevation';
s.Apodrx=[0,0];sdoc.Apodrx='Receiver apodization azimuth, elevation (beta values for tukeywin window)';
s.f0rx=4.0e6;sdoc.f0rx='centerfrequency receiver filter';
s.Brx=0.3e6;sdoc.Brx='bandwidth receiver filter';

s.range=[30,30,10]*1e-3;sdoc.range='simulation range [m] in x, y, and z direction';
s.resolution=[0.25,0.25,0.1]*1e-3;sdoc.resolution='simulation point distance [m] in x, y, and z direction';

s.sphericalCorrection=0;sdoc.sphericalCorrection='Spherical correction of of field in focal plane =1 enable, =0 disable';
s.dynRange=50;sdoc.dynRange='Dynamic range [dB] for plot function ';
s.pwr2=1; sdoc.pwr2='the number of frequency points are increased to nextpow2 to increase fft computation';
s.betaN=3.5; sdoc.betaN='Nonlinear coeficient betaN= 1+ B/(2*A).  See Szabo ch. 12';
s.kappa=452e-12; sdoc.kappa='Compressibility';
s.ro=988; sdoc.ro='Density';
s.p1maxF=1e6; sdoc.p1maxF='Peak pressure in focal plane';
s.a=0; sdoc.a='Attenuation at 1 MHz in Np/m (>0)';
s.b=0; sdoc.b='Attenuation frequency power -> att in Np/MHz^b/m';


%%%%%%%%%%%%%%%%%%    end of defaultsettings   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

function res=reCalc(Item,val,val2)
% reCalc  -  recalculate any simulation data
global pSettings pData pDataDoc;
if nargin<3, val2=[];end;
if nargin<2, val=[];end;
s=pSettings;
c=s.c;
res=[];
switch Item,
    
case {'axes'}
    calcAxes(s);
case {'aperturetx'}
    xax=Propose3('get','xax');yax=Propose3('get','yax');
    if ~isempty(xax),
        if strcmp(class(val),'char'),pSettings.Ashapetx=val;end;
        pData.Atx=createAperture(pSettings.Ashapetx,s.Dtx(1),s.Dtx(2),xax,yax); 
        pDataDoc.Atx='Aperturefunction transmit';
    end;
case {'aperturerx'}
    xax=Propose3('get','xax');yax=Propose3('get','yax');
    if ~isempty(xax),
        if strcmp(class(val),'char'),pSettings.Ashaperx=val;end;
        pData.Arx=createAperture(pSettings.Ashaperx,s.Drx(1),s.Drx(2),xax,yax);
        pDataDoc.Arx='Aperturefunction receive';
    end;
case {'txpulse'}
    df=c/2/s.range(3);
    Nf=round(s.Btx/df);
    pData.f1=linspace(s.f0tx-s.Btx/2,s.f0tx+s.Btx/2,Nf);%df=pData.f1(2)-pData.f1(1);
    pDataDoc.f1='Frequency axis transmit';
   %pData.P=trondwin(length(pData.f1),1.5)';%Trond W. variant
    pData.P=gausswin(length(pData.f1),1.5)';%Gauss variant, spring 2014 HS.
    power=df*sum(pData.P.^2);
    pDataDoc.P='pulse frequency response transmit';
    res=pData.P;
case {'rxfilter'}
    df=c/2/s.range(3);
    Nf=round(s.Brx/df);
    pData.f2=linspace(s.f0rx-s.Brx/2,s.f0rx+s.Brx/2,Nf);
    pDataDoc.f2='Frequency axis receive';
    pData.Hrx=gausswin(length(pData.f2),3.5)';%
    %pData.Hrx=tukeywin(length(pData.f2),0.5)';%
    pDataDoc.Hrx='receiver filter frequency response';
    res=pData.Hrx;
case {'txfieldfocus'}
    if isempty(Propose3('get','P')),reCalc('txpulse');end;
    if isempty(Propose3('get','Atx')),reCalc('aperturetx');end;
    [P11,fx1,fy1]=fieldFoc(pData.f1,pData.P,s.Dtx,s.Rtx,s.Apodtx,s.range,s.sphericalCorrection,s.Ashapetx,s.p1maxF);
    pData.P11=P11; pDataDoc.P11='Linear field in focal point z1 in Fourier domain';
    pData.fx1=fx1;pDataDoc.fx1='Frequency axis spatial x';
    pData.fy1=fy1;pDataDoc.fy1='Frequency axis spatial y';
    pData.z1=mean(s.Rtx);pDataDoc.z1='mean ROC tx aperture';
    res=P11;
case {'rxfield'}
    reCalc('rxfilter');
    s=pSettings;
    if isempty(Propose3('get','Arx')),reCalc('aperturerx');end;
    [Prx,fx2,fy2]=fieldFoc(pData.f2,pData.Hrx,s.Drx,s.Rrx,s.Apodrx,s.range,s.sphericalCorrection,s.Ashaperx);
    pData.Prx=Prx;pData.fx2=fx2;pData.fy2=fy2;
    pData.z1=mean(s.Rtx);
    res=Prx;
case {'aperspec'}
    [P11,fx1,fy1]=fieldFT(pData.f1,pData.P,s.Dtx,s.Rtx,s.range,s.p1maxF,s.rx,s.ry,s.prof);
    pData.P11=P11; pDataDoc.P11='Linear field in focal point z1 in Fourier domain';
    pData.fx1=fx1;pDataDoc.fx1='Frequency axis spatial x';
    pData.fy1=fy1;pDataDoc.fy1='Frequency axis spatial y';
    pData.z1=val;pDataDoc.z1='mean ROC tx aperture';
    res=P11;
case {'propLin'}
    P11=Propose3('get','P11');
    if isempty(P11), 
      reCalc('txfieldfocus');
    end;
    fx1=pData.fx1;fy1=pData.fy1;f1=pData.f1;P11=pData.P11;
    pData.z21=val;pDataDoc.z21='Observation point linear wave';
    P12=propLin(P11,fx1,fy1,f1,pData.z1,pData.z21,s.a,s.b);
    pData.Ptx=P12;
    [p12,xax,yax,zax]=fourier2spaceTime(P12,fx1,fy1,f1,s.resolution);
    pData.p12=p12;pDataDoc.p12='Linear field in observation point z21';
    pData.xax=xax;pDataDoc.xax='x axis for linear/nonlinear field';
    pData.yax=yax;pDataDoc.yax='y axis for linear/nonlinear field';
    pData.zax=zax;pDataDoc.zax='z axis for linear/nonlinear field';
    if isempty(Propose3('get','Atx')),reCalc('aperturetx');end;    
    if isempty(Propose3('get','Arx')),reCalc('aperturerx');end;
    res=p12;
case {'prop2harm'}
    P11=Propose3('get','P11');
    if isempty(P11), reCalc('txfieldfocus');end;
    fx1=pData.fx1;fy1=pData.fy1;f1=pData.f1;P11=pData.P11;
    pData.z22=val;
    [P22,fx2,fy2,f2]=prop2harm(P11,fx1,fy1,f1,pData.z1,pData.z22,s.a,s.b);
    pData.P22=P22;pData.fx2=fx2;pData.fy2=fy2;pData.f2=f2;
    pData.Ptx=P22;
    [p22,xax,yax,zax]=fourier2spaceTime(P22,fx2,fy2,f2,s.resolution);
    pData.p22=p22;pDataDoc.p22='Second harmonic field in observation point';
    pData.xax=xax;pDataDoc.xax='x axis for linear/nonlinear field';
    pData.yax=yax;pDataDoc.yax='y axis for linear/nonlinear field';
    pData.zax=zax;pDataDoc.zax='z axis for linear/nonlinear field';
    if isempty(Propose3('get','Atx')),reCalc('aperturetx');end;    
    if isempty(Propose3('get','Arx')),reCalc('aperturerx');end;
    res=p22;
case {'2wayResponse'}
    xax=Propose3('get','xax');
    yax=Propose3('get','yax');
    Pfrx=Propose3('get','Pfrx');
    if isempty(Pfrx),
        pSettings.Rrx(1)=val;%dynamic azimuth focus receive
        reCalc('rxfield');
        [Pfrx,xax,yax]=fourier2space(pData.Prx,pData.fx2,pData.fy2,s.resolution);
        pData.Pfrx=Pfrx;
    end;
    Pftx=Propose3('get','Pftx');
    if isempty(Pftx),
        Ptx=Propose3('get','Ptx');
        if isempty(Ptx), disp('Tx field is missing');return;end;
        [Pftx,xax,yax]=fourier2space(Ptx,pData.fx2,pData.fy2,s.resolution);
    end;
    deltax=0;deltay=0;
    if length(val2)>0,
      
      %Bugfix. Torbj.
      val2 = val2{1};
      
      deltax=round(val2(1)/s.resolution(1));
    end;
    if length(val2)>1,deltay=round(val2(2)/s.resolution(2));end;

    %Torbj.
    %fixing so negative deltax'es are OK; wraparound a few elements of Pfrx.
    PfrxExtended = Pfrx([end-abs(deltax)+1:end, 1:end],[end-abs(deltay)+1:end, 1:end],:);
    PfrxToobig=PfrxExtended(abs(deltax)+1+deltax:end,abs(deltay)+1+deltay:end,:); 

    %zeropadding, in case toobig is too small... (positive steering and such)
    PfrxToobig(size(Pftx,1),size(Pftx,2),size(Pftx,3))=0;

    %cropping to correct size:
    PfrxS=PfrxToobig(1:size(Pftx,1),1:size(Pftx,2),1:size(Pftx,3));

    pData.Pf2w=PfrxS.*Pftx;    
    pDataDoc.Pf2w=['2-way profile. First two dimensions are spatial (x and y), third dimension (z or time)', ...
                   'in k-space.'];
    pData.xax=xax;pData.yax=yax;
    res=pData.Pf2w;

 end;%switch lower(action)

%%%%%%%%%%%%%%%%%%    end of reCalc   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

function w=trondwin(N,beta)
x=linspace(-beta,beta,N);
w=exp(-x.^4);

function w=gausswin(N,foo)
sigma = N/5;
tenv = linspace(-N/2,N/2,N);
w = exp(-tenv.^2'/(2*sigma^2));

function  calcAxes(s)
%calulate frequency and spatio tempral axes from parametersettings s
    df=c/2/s.range(3);
    Nf=round(s.Btx/df);
    pData.f1=linspace(s.f0tx-s.Btx/2,s.f0tx+s.Btx/2,Nf);
    pData.P=gausswin(length(pData.f1),2.5)';%pulse frequency response
    res=pData.P;
% to be finished....

function P12=propLin(P11,fx1,fy1,f1,z1,z2,a,b)
% propLin: Linear propagation by angular spectrum method
% P11 is the Fouriertransform of the field in pos. z1, with freq. axes fx1,fy1,f1
% P12 is the Fouriertransform of the field in pos. z2, with freq. axes fx1,fy1,f1
c=speedOfSound;
f0=xmitfreq;
cw=dispersion(f1,f0,a,b);
K=katet(fx1,fy1,f1,a,b,c);
% FP - Sep 2011. commented out to allow for attenuation. if active, drop evanescent waves
% Iblank=find(abs(imag(K)));P11(Iblank)=0;
P12=P11.*exp(-1i*(z2-z1)*K);





%%%%%%%%%%%%%%%%%%    end of propLin   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sqt1=katet(fx,fy,f,a,b,c)
% to be used in forward, backward propagation, angular spectrum method
sqt1=zeros(length(fx),length(fy),length(f),calcformat);
for k=1:length(f),
    fxm=fx'*ones(1,length(fy));
    fym=ones(length(fx),1)*fy;
    sqt1(:,:,k)=sqrt(f(k)^2-fxm.^2-fym.^2);
    % FP - Sep 2011. Immportant change here. With our sign convention, the
    % imaginary part of K should be negative to ensure that solution does
    % not diverge when z-> infty. By default Im(sqrt(a)) > 0 when a <0 in
    % Matlab
    sqt1(:,:,k) = real(sqt1(:,:,k))-1i*sign(sqt1(:,:,k)).*imag(sqt1(:,:,k));   
    % FP - Sep 2011. Add power law attenuation here
    sqt1(:,:,k) = complex(2*pi/c*(sqt1(:,:,k) -f(k))-1i*a*(f(k)/10^6)^b);
    
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% In most cases dispersion does not make a difference. This is not used.
function cw=dispersion(f,f0,a,b)
c=speedOfSound; % assumed sound speed at center frequency f_0
if a==0 || b== 0 || b==2    % no disperion in those cases
    cw = c*ones(size(f));
else % FP - Sep 2011. Kramers-Kronig relation giving c(w)-c_0 as a function of attenuation law is used here
    cw_inv = 1/c + a/(10^6)^b*tan(pi*b/2)*((2*pi*f).^(b-1)-(2*pi*f0)^(b-1));
    cw = 1./cw_inv;
end

%%%%%%%%%%%%%%%%%    end of dispersion   %%%%%%%%%%%%%%%%%%%%%%%%%%%%


% % %%%%%  Commented out to make room for the MEX-version of this function Dec. 2013 Haakon Seljaasen
% 
% function [P22,fx2,fy2,f2]=prop2harm(P11,fx1,fy1,f1,z1,z2,a,b)
% %% prop2harm: NonLinear propagation of 2.harmonic wave quasilinear method
% % P11 is the Fouriertransform of the field in pos. z1, with freq. axes fx1,fy1,f1
% % P22 is the Fouriertransform of the 2. harm field in pos. z2, with freq. axes fx1,fy1,f1
% useSymmetry=symmetryChoice;
% c=speedOfSound;
% [Nx,Ny,Nf]=size(P11);
% %save('At start of prop2harm');
% dfx=(fx1(2)-fx1(1));
% dfy=(fy1(2)-fy1(1));
% df=(f1(2)-f1(1));
% %frequency axes for 2.harmonic
% f0=squeeze(mean(f1));
% f2=f1+f0;Nf2=length(f2);
% % fx2= 2*fx1(1):dfx:(2*fx1(end)+1);Nx2=length(fx2);
% % fy2= 2*fy1(1):dfy:(2*fy1(end)+1);Ny2=length(fy2);
% % FP - Sep 2011. This must have been a typo.
% fx2= 2*fx1(1):dfx:(2*fx1(end)+dfx);Nx2=length(fx2);
% fy2= 2*fy1(1):dfy:(2*fy1(end)+dfy);Ny2=length(fy2);
% 
% % cw=dispersion(f2,2*f0,a,b);
% K=katet(fx1,fy1,f1,a,b,c);
% [ro,kappa,betaN]=acousticalProperties;
% %***************** original solution  *****************%
% % Cfactor=betaN*kappa*4*pi^2/(1i*c^2)*8*pi^3/c^2;%frequency indep. part of C
% %***************** corrected solution *****************%
% Cfactor=-1i*betaN*kappa*pi^2/c^4;%frequency indep. part of C
% %******************************************************%
% 
% P22=zeros(Nx2,Ny2,Nf2,calcformat);
% P22b=zeros(Nx2,Ny2,Nf2,calcformat);
% Nm=Nf2;Nk=Nx2;Nn=Ny2;
% runner=0;
% if useSymmetry, Nk=round(Nk/2);Nn=round(Nn/2);end;
% Nf_2=fix(Nf/2);
% waitbar_sh(Nm);
% for m=1:Nm,
%     f=f2(m);
%     fStart=max(1,1+m-Nf_2);
%     fStop=min(Nf,m-Nf_2+Nf-1);
%     If1=max(1,1+m-Nf_2):min(Nf,m-Nf_2+Nf-1);
%     If2=m+1+Nf_2-If1;
%     for k=1:Nk,
%         fx=fx2(k);
%         fxStart=max(1,1+k-Nx);
%         fxStop=min(Nx-1,k);
%         Ix1=max(1,1+k-Nx):min(Nx-1,k);
%         Ix2=1+k-Ix1;
%         P221=zeros(1,Nn);
%         P221b=zeros(1,Nn);
%         for n=1:Nn,
%             fy=fy2(n);
%             fyStart=max(1,1+n-Ny);
%             fyStop=min(Ny-1,n);
%             Iy1=max(1,1+n-Ny):min(Ny-1,n);
%             Iy2=1+n-Iy1;
%             %Skriver om matlabstruktur til for-lkker:
%             
%             Kf=katet(fx,fy,f,a,b,c);%K(f) in equation(11)Moved into
%             %nodePointCalculation
%           if  ~(isempty(Ix1)||isempty(Iy1)||isempty(If1)) && isreal(Kf)
%                 Gamma=-Kf+K(Ix1,Iy1,If1)+K(Ix2,Iy2,If2);                
%                 Hp=exp(1i*Gamma*(z1-z2/2)).*sinc(-Gamma*z2/2/pi);
%                 Integrand=P11(Ix1,Iy1,If1).*P11(Ix2,Iy2,If2);
%                 Integrand=Integrand.*Hp;
%                 sumI=sum(Integrand(:));
%                 sumI=z2*f^2*exp(-1i*Kf*(z2-z1))*df*dfx*dfy*sumI;
%                 C=Cfactor/(Kf+2*pi*f/c);
%                 %C=1;
%                 P221(n)=C*sumI;
%             end;
%         end;%for n=1:length(fy2),
%         if useSymmetry,
%             P22(k,1:Nn,m)=P221;
%             P22(Nx2-k+1,1:Nn,m)=P221;
%             P22(Nx2-k+1,Ny2-(1:Nn)+1,m)=P221;
%             P22(k,Ny2-(1:Nn)+1,m)=P221;
%         else
%             P22(k,:,m)=P221;
%             P22b(k,:,m)=P221b;
%         end
%         
%     end;%for k=1:length(fx2),
%       
%     waitbar_sh;
% end;%for m=1:length(f2),

%%%%%%%%%%%%%%%%%%    end of prop2harm   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% fourier2spaceTime performs zeropadding and conversion to space-time by
% ifft
function [p12,xax,yax,zax]=fourier2spaceTime(P12,fx1,fy1,f1,resolution)
% P12 is the Fouriertransform of the field in pos. z2, with freq. axes fx1,fy1,f1
% p22 is the 2.harmonic pulsed wave field in position z2, with spatial axes xax,yax,zax=tax*c/2
% resolution [dx,dy,dz] simulation point distance [m] in x, y, and z direction

% 2011.05.10  Mult by df*dfx*dfy
c=speedOfSound;

dfx=(fx1(2)-fx1(1));
dfy=(fy1(2)-fy1(1));
df=(f1(2)-f1(1));

% Nfx=2*round(c/dfx/resolution(1)/2)+1;
% Nfy=2*round(c/dfy/resolution(2)/2)+1;
% Nfs=2*round(c/2/df/resolution(3)/2)+1;
% Need more extrapolating, extending zero padding HS. 2014
Nfx=3*round(c/dfx/resolution(1)/2)+1;
Nfy=3*round(c/dfy/resolution(2)/2)+1;
Nfs=3*round(c/2/df/resolution(3)/2)+1;


%symmetrical zeropadding in fx,fy,f
[Nx,Ny,Nf]=size(P12);
%Nfx=Nx;Nfy=Ny;Nfs=Nf;

xax=linspace(-1/dfx*c/2,1/dfx*c/2,Nfx);
yax=linspace(-1/dfy*c/2,1/dfy*c/2,Nfy);
% zax=linspace(-1/df*c/4,1/df*c/4,Nfs);
% FP - Sept 2011. Don't think this was correct
zax=linspace(-1/df*c/2,1/df*c/2,Nfs);

P12zp=zeros(Nfx,Nfy,Nfs,calcformat);
ix1=1+round(1+(Nfx-1)/2-(Nx-1)/2);
iy1=1+round(1+(Nfy-1)/2-(Ny-1)/2);
 if1=round(1+(Nfs-1)/2+1+f1(1)/df);
% Haakon Seljaasen - April 2014. We need the frequency offset to compare phase of
% the puls to Abersim: Sept. 2011 edit is reset.
%if1=round(1+(Nfs-1)/2+1-(Nf-1)/2); 
% FP - Sept 2011. The freq. offset is dropped. The result is the envelope
% of the pulse. That's fine since we know what is in the envelope.
% This give the same zax for p1 og p2 (more practical).
 
P12zp(ix1:ix1+Nx-1,iy1:iy1+Ny-1,if1:if1+Nf-1)=P12;
[Nxzp,Nyzp,Nfzp]=size(P12zp);
P12zp=fftshift(P12zp);
p12=ifftn(P12zp);
p12=fftshift(p12);
p12=p12*dfx/c*dfy/c*df*Nxzp*Nyzp*Nfzp;
if testPlot,
    figure(8);
    f0=squeeze(mean(f1));
    subplot(2,1,1);imagesc(abs(squeeze(P12zp(:,1,:))));pause(0.01);
    if2=round(1+f0/df);
    subplot(2,1,2);imagesc(abs(squeeze(P12zp(:,:,if2))));pause(0.01);
    p12y=squeeze(p12(:,round((Nfy+1)/2),:));
    logp=20*log10(abs(p12y));
    logp=logp-max(logp(:));
    dynRange=50;
    figure(10);image(zax,xax,63/dynRange*logp+63);axis('image');
    figure(9);clf;
    p12x=squeeze(p12y(:,(Nfs+1)/2));
    plot(xax,abs(p12x),xax,real(p12x),xax,imag(p12x));
end;
%%%%%%%%%%%%%%%%%%    end of fourier2spaceTime   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [p,xax,yax]=fourier2space(P,fx1,fy1,resolution)
% fourier2space performs zeropadding and conversion to space-frequency by
% ifft of the first two dimensions of P.
% P is the Fouriertransform of the field in pos. z2, with freq. axes fx1,fy1,f1
% p22 is the 2.harmonic pulsed wave field in position z2, with spatial axes xax,yax,zax=tax*c/2
% resolution [dx,dy,dz] simulation point distance [m] in x, y, and z direction
% 2011.05.10  Mult by df*dfx*dfy
c=speedOfSound;
Nfx=2*round(c/(fx1(2)-fx1(1))/resolution(1)/2)+1;
Nfy=2*round(c/(fy1(2)-fy1(1))/resolution(2)/2)+1;
dfx=(fx1(2)-fx1(1));
dfy=(fy1(2)-fy1(1));
xax=linspace(-1/dfx*c/2,1/dfx*c/2,Nfx);
yax=linspace(-1/dfy*c/2,1/dfy*c/2,Nfy);
%symmetrical zeropadding in fx,fy
[Nx,Ny,Nf]=size(P);
Pzp=zeros(Nfx,Nfy,Nf,calcformat);
ix1=1+round(1+(Nfx-1)/2-(Nx-1)/2);
iy1=1+round(1+(Nfy-1)/2-(Ny-1)/2);
Pzp(ix1:ix1+Nx-1,iy1:iy1+Ny-1,:)=P;
[Nxzp,Nyzp]=size(Pzp);
Pzp=fftshift2(Pzp);
p=ifft2(Pzp);
p=fftshift2(p);
p=p*dfx/c*dfy/c*Nxzp*Nyzp;

function y=fftshift2(x)
Nf=size(x,3);
for n=1:Nf,
    y(:,:,n)=fftshift(x(:,:,n));
end;
%%%%%%%%%%%%%%%%%%    end of fourier2space   %%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [P1,fx,fy]=fieldFoc(f,P,D,Rxy,Apod,range,sphericalCorrection,shape,p1maxF,rx,ry,prof)
% spatial/temporal Fourier transform of pulsed field in focal plane
%fx,fy,f frequency axis in x,y,t
% P pulse frequency response
% Dx,Dy xducer aperture in x and y direction
% R=[Rx,Ry] focal depth for x and y
% betax, betay parameters for tukey window apodization 
oversampling=2;
ROCcorrection=1;
if nargin<8,shape='rectangular';end;
e=[];
if (length(shape)>7) & (shape(1:8)=='elliptic'), %superelliptic form of active elements in matrix a2d
    e=2.0;
    if length(shape)>9,e=str2num(shape(10:end));end;
end;
c=speedOfSound;
R=mean(Rxy);
Dx=D(1);
Dy=D(2);
circsymm=(e==2)&(Dx==Dy);
betax=Apod(1);
betay=Apod(2);
fmax=1.1*f(end);
Nx=1*(1+2*round(range(1)*fmax*Dx/R/c)); % rangex=Nx/fxmax*c=Nx*c/(2*fmax*Dx/R)=R*Nx*c/2/fmax/Dx
Ny=1*(1+2*round(range(2)*fmax*Dy/R/c)); 
fx=linspace(-fmax*Dx/R,fmax*Dx/R,Nx);
fy=linspace(-fmax*Dy/R,fmax*Dy/R,Ny);
fxi=linspace(-fmax*Dx/R,fmax*Dx/R,1+oversampling*(Nx-1));
fyi=linspace(-fmax*Dy/R,fmax*Dy/R,1+oversampling*(Ny-1));
dfx=fx(2)-fx(1);dfy=fy(2)-fy(1);df=f(2)-f(1);
% dx=c/dfx/length(fx);dy=c/dfy/length(fy);
fxm=fxi'*ones(1,length(fyi));
fym=ones(length(fxi),1)*fyi;
% ft=max(abs(ifft(fftshift(P)))*df*length(f));
currfig=gcf;
%hlp2=ones(oversampling+1,oversampling+1)/(oversampling+1)^2;%2D lpfilter
hlp2=ones(oversampling,oversampling)/(oversampling)^2;%2D lpfilter

P1=zeros(length(fx),length(fy),length(f),calcformat);
P1i0=zeros(length(fxi),length(fyi),calcformat);

for k=1:length(f),
    fxmax=0.5*Dx/R*f(k);    % factor 0.5 because Dx/Dy are diameters
    fymax=0.5*Dy/R*f(k);
    Ix=find(abs(fxi)<fxmax);
    Iy=find(abs(fyi)<fymax);
    wx=tukeywin(length(Ix),betax);%wx(1)=0.5*wx(1);wx(end)=0.5*wx(end);
    wy=tukeywin(length(Iy),betay);%wy(1)=0.5*wy(1);wy(end)=0.5*wy(end);
    w=wx*wy';
    
    if circsymm, w=tukey2(length(Ix),betax);end;
    P1k=P1i0;
    %     P1k(Ix,Iy)=w*P(k)/f(k);   % Why is there a /f(k) ?
    P1k(Ix,Iy)=w*P(k);
    if ~isempty(e),%superelliptic aperture
        Iblank=find((abs(fxm/fxmax).^e+abs(fym/fymax).^e)>1);
        P1k(Iblank)=0;
    end;
    %ka=0.25;b=[ka,1,ka];b=b'*b;P1k=filter2(b,P1k);
    if oversampling>1, P1k=filter2(hlp2,P1k); end;
    if 0,
        figure(7);
        subplot(2,2,1);imagesc(abs(P1k));axis('image');colormap(gray);grid;pause;
    end;
    
    if ROCcorrection,
        Ha=apertureCorrection(R,Rxy(1),Rxy(2),f(k),fxi,fyi);
        P1k=P1k.*Ha;
    end;
    if sphericalCorrection,
        Hs=focalcorrection(R,f(k),fxi,fyi);
        P1u=P1k;
        %Hs=Hs(1:end-2,1:end-2);%tja....
        P1k=conv2(P1k,Hs,'same');
        if testPlot,
            figure(7);
            subplot(2,2,1);imagesc(abs(P1u));axis('image');colormap(gray);grid;
            subplot(2,2,3);imagesc(abs(P1k));axis('image');
            subplot(2,2,2);imagesc(abs(Hs));axis('image');
            subplot(2,2,4);imagesc(abs(ifft(fftshift(P1k))));axis('image');
            pause;
        end;
    end;%sphericalCorrection
    P1k=P1k(1:oversampling:end,1:oversampling:end);%decimation to correct sampling rate
    P1(:,:,k)=P1k;
end;%k=1:length(f),

% FP - Sept 2011. Find coefficient to scale P1 so that max of temporal
% pulse at focus is p1maxF.
F=max(max(max(abs(fftshift(ifftn(ifftshift(P1))*dfx/c*dfy/c*df*Nx*Ny*length(f))))));
P1=P1/F*p1maxF;
if testPlot,if ~isempty(currfig),figure(currfig);end;end;


function w=tukey2(N,R)
Nl=512;
ic=round(N/2);
x=ones(N,1)*linspace(-1,1,N);
y=linspace(-1,1,N)'*ones(1,N);
r=sqrt(x.^2+y.^2);
Ir=find(r<1);
wN=tukeywin(Nl,R);
w=0*x;
w(Ir)=wN(1+round(0.49*Nl*(1+r(Ir))));
%%%%%%%%%%%%%%%%%%    end of fieldFoc   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Hs=focalcorrection(R,f0,fx,fy)
% correction of phase due to curvature of focal "plane" for single frequency f0
c=speedOfSound;
dfx=fx(2)-fx(1);dfy=fy(2)-fy(1);
xmax=1/dfx*c/2;ymax=1/dfy*c/2;
xax=linspace(-xmax,xmax,length(fx))-3.5*xmax/length(fx);%gjengs konstant trukket fra
yax=linspace(-ymax,ymax,length(fy))-2.5*ymax/length(fy);%for  sentrere nrfelt respons
x=xax'*ones(1,length(yax));
y=ones(length(xax),1)*yax;
% tau=(R-sqrt(R^2-x.^2-y.^2))/c;
% FP - Sept 2011. Change to parabolic correction according to theory when
% using Fraunhoffer approximation. Should make very little difference when
% x and y << R
tau=(x.^2+y.^2)/2/R/c;
h=exp(-1i*2*pi*f0*tau);
tind=0.8;
w=tukeywin(length(xax),tind)*tukeywin(length(yax),tind)';
h=h.*w;
h=fftshift(h);
Hs=fft2(h);
Hs=fftshift(Hs);

function A=createAperture(shape,Dx,Dy,xax,yax)
x=xax'*ones(1,length(yax));
y=ones(length(xax),1)*yax;
A=zeros(size(x),calcformat);
if isempty(shape),shape='rectangular';end;
switch shape(1:8),
    case {'rectangu','rectangular'}
        A=ones(size(A));
        A(find(abs(x)>Dx/2))=0;
        A(find(abs(y)>Dy/2))=0;
    case {'elliptic'} %superelliptic form of active elements in matrix a2d
        e=2.0;
        if length(shape)>9,e=str2num(shape(10:end));end;
        if isempty(e),display('error superelliptic command');return;end;
        elem=find((abs(x/Dx*2).^e+abs(y/Dy*2).^e)<1);
        A(elem)=1;
    otherwise disp('Propose unknown aperture shape');
end;


function Ha=apertureCorrection(R,Rx,Ry,f,fxv,fyv)
% correction of aperture phase due to deviation from ideal spherical surface
% R: radius of curvature (ROC) for ideal spherical surface
% Rx, Ry:  ROC in azimuth, elevation.
% Assumption: curvature in x and y are added
% f frequency (single frequency)
% fx, fy azimuth, elevaton spatial frequency vectors
% Denne M sjekkes nye!!!!!!!!!!!!
c=speedOfSound;
fx=fxv'*ones(size(fyv));
fy=ones(size(fxv'))*fyv;
fx2=(fx/f).^2;
fy2=(fy/f).^2;
delta=R*(1-sqrt(1-fx2-fy2)) - Rx*(1-sqrt(1-(R/Rx)^2*fx2)) - Ry*(1-sqrt(1-(R/Ry)^2*fy2));
% Ha=exp(-0.5*1i*2*pi*f/c*delta);%Hvorfor i ??????????? m man justere med
% 0.5 faktor???????
% FP - Sept 2011. Removed the 0.5 factor. Should not be here in theory.
% Could not find justification with data as why it should be here.
Ha=exp(-1i*2*pi*f/c*delta);
if testPlot, figure(7);plot(delta);pause(0.1);end;

%%%%%%%%%%%%%%%%%%   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotResults(option,figNo)
if nargin<2,option=[];end;
if nargin<1,figNo=[];end;
%if isempty(figNo),figNo=1;end;
global pSettings pData;
s=pSettings;
C=speedOfSound;
dynRange=s.dynRange;
if isempty(option),option='default';end;
if ~isempty(figNo);figure(figNo);end;
cmap = 'gray';

switch option,
    case {'default','2w','2wprofile'}
        if ~isempty(Propose3('get','Pf2w')),
            x=1e3*pData.xax;y=1e3*pData.yax;
            p2w=squeeze(sum(abs(pData.Pf2w).^2,3));
            dynRange=s.dynRange;
            p2wxy=10*log10(p2w'+1e-30); %Torbj. Avoiding log of zero
            %image(x,y,63+63/dynRange*(p2wxy-max(p2wxy(:))));axis image;
            p2max=max(p2wxy(:));
            pcolor(x,y,p2wxy-p2max);axis image;caxis([-dynRange,0]);
            if strcmp(option,'2wprofile'),
                plot(y,p2wxy(:,round(end/2)));
                [Nx,Ny]=size(p2w);
                Nx2=(Nx-1)/2;
                Ny2=(Ny-1)/2;
                N=min([Nx2,Ny2]);
                dx=x(2)-x(1);
                for n=0:N-1,
                    prof(n+1)=sum(p2w(Nx2+1-n:Nx2+1+n,Ny2+1-n))+sum(p2w(Nx2+1-n,Ny2+1-n:Ny2+n+1));
                end;
                rprof=dx*(0:N-1);
                prof=prof/max(prof);
                cprof=cumsum(prof(end:-1:1));cprof=cprof(end:-1:1);
                cprof=cprof/max(cprof);
                subplot(1,2,2);plot(rprof,10*log10(prof),rprof,10*log10(cprof));grid;
            end;
        end;
case {'field'}
if ~isempty(Propose3('get','p12')),
    x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
    p1xy=double(10*log10(mean(abs(pData.p12).^2,3))');
    p1xz=double(20*log10(max(0,abs(squeeze(pData.p12(:,round(end/2),:)))))');
    subplot(3,2,1);
    %image(x,y,63+63/dynRange*(p1xy-max(p1xy(:))));axis image;
    pcolor(x,y,(p1xy-max(p1xy(:))));axis image;,caxis([-dynRange,0]);shading('interp')
    title(['Fundamental z= ',num2str(1e3*pData.z21),' mm']);
    ylabel('Elevation [mm]');
    subplot(3,2,3);
    %image(x,z,63+63/dynRange*(p1xz-max(p1xz(:))));axis image;
    pcolor(x,z,p1xz-max(p1xz(:)));axis image;caxis([-dynRange,0]);shading('interp')
    ylabel('Axial [mm]');
    subplot(3,2,5);
    xprofile=squeeze(p1xy(round(end/2),:))-max(p1xy(:));
    plot(x,xprofile);axis([x(1),x(end),-dynRange,0]);grid;
    xlabel('Azimuth [mm]');
   %figure(7);contour(x,y,p1xy-max(p1xy(:)),[-dynRange:3:0]);axis image;

end;
if ~isempty(Propose3('get','p22')),
    x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
    %x=pData.xax;y=pData.yax;z=pData.zax;
    p2xy=double(10*log10(mean(abs(pData.p22).^2,3)))';
    p2xz=double(20*log10(abs(squeeze(pData.p22(:,round(end/2),:)))))';
    subplot(3,2,2);
    %image(x,y,63+63/dynRange*(p2xy-max(p2xy(:))));axis image;
    pcolor(x,y,(p2xy-max(p2xy(:))));axis image;,caxis([-dynRange,0]);shading('interp');
    title(['Second harmonic. z= ',num2str(1e3*pData.z22),' mm']);
    ylabel('Elevation[mm]');
    subplot(3,2,4);
    %image(x,z,63+63/dynRange*(p2xz-max(p2xz(:))));axis image;
    z2=linspace(z(1),z(end),size(p2xz,1));%hack to get correct z-axis
    pcolor(x,z2,p2xz-max(p2xz(:)));axis image;caxis([-dynRange,0]);shading('interp')
    ylabel('Axial [mm]');
    subplot(3,2,6);
    xprofile=squeeze(p2xy(round(end/2),:))-max(p2xy(:));
    plot(x,xprofile);axis([x(1),x(end),-dynRange,0]);grid;
    xlabel('Azimuth [mm]');  
    %figure(8);contour(x,y,p2xy-max(p2xy(:)),[-dynRange:3:0]);axis image;grid;

end;
disp('Propose settings:');disp(s);
    case {'phase'} %Added by HS - 2014.
        if ~isempty(Propose3('get','p12')),
            x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p1zx=double(real(squeeze(pData.p12(:,round(end/2),:))));
            p1zx=TodB(p1zx);
            p1zy=double(real(squeeze(pData.p12(round(end/2),:,:))));
            p1zy=TodB(p1zy);
            if ~isempty(Propose3('get','p22')),
                subplot(2,2,1);     %Original Propose plot, HS 2014.
            else
                subplot(2,1,1);    %Original Propose plot, HS 2014.
            end;
            
            %surf(t,x,p1zx);axis image;caxis([-dynRange,dynRange]);shading('interp');colorbar;
            %pcolor(t,x,p1zx);caxis([-dynRange,dynRange]);shading('interp');colorbar;
            imagesc(t,x,p1zx);caxis([-dynRange,dynRange]);shading('interp');colormap(cmap);
            xlim([-5 5]);ylim([-20 20]);
            ylabel('Azimuth [mm]');
            xlabel('Retarded time [\mu s]');
            title(['Propose fundamental z= ',num2str(1e3*pData.z21),' mm']);
            if ~isempty(Propose3('get','p22')),
                subplot(2,2,3);         %Original Propose plot, HS 2014.
            else
                subplot(2,1,2);    %Original Propose plot, HS 2014.
            end;
            %surf(t,y,p1zy);axis image;caxis([-dynRange,dynRange]);shading('interp');colorbar;
            %pcolor(t,y,p1zy);caxis([-dynRange,dynRange]);shading('interp');colorbar;
            imagesc(t,y,p1zy);caxis([-dynRange,dynRange]);shading('interp');colormap(cmap);
            xlim([-5 5]);ylim([-20 20]);
            ylabel('Elevation [mm]');
            title(['Fundamental z= ',num2str(1e3*pData.z21),' mm']);
            
            xlabel(['Retarded time [\mu s]']);
        end;
        if ~isempty(Propose3('get','p22')),
            x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p2zx=double(real(squeeze(pData.p22(:,round(end/2),:))));
            p2zx=TodB(p2zx);
            p2zy=double(real(squeeze(pData.p22(round(end/2),:,:))));
            p2zy=TodB(p2zy);
            
            subplot(2,2,4);         %Original Propose plot, HS 2014.
            %surf(t,y,p2zy);axis image;caxis([-dynRange,dynRange]);shading('interp');
            %pcolor(t,y,p2zy);axis image;caxis([-dynRange,dynRange]);shading('interp');colorbar;
            imagesc(t,y,p2zy);caxis([-dynRange,dynRange]);shading('interp');colormap(cmap);%colorbar
            xlim([-5 5]);ylim([-20 20]);
            title(['Second harmonic. z= ',num2str(1e3*pData.z22),' mm']);
            ylabel('Elevation [mm]');
            xlabel('Retarded time [$\mu s$]');
            
            subplot(2,2,2);         %Original Propose plot, HS 2014.
            %surf(t,x,p2zx);axis image;caxis([-dynRange,dynRange]);shading('interp');
            %pcolor(t,x,p2zx);axis image;caxis([-dynRange,dynRange]);shading('interp');colorbar;
            imagesc(t,x,p2zx);caxis([-dynRange,dynRange]);shading('interp');colormap(cmap);%colorbar
            title(['Second harmonic. z= ',num2str(1e3*pData.z22),' mm']);
            xlim([-5 5]);ylim([-20 20]);
            ylabel('Azimuth [mm]');
            xlabel('Retarded time [$\mu s$]');
        end;
    case {'signal'} %Added by HS - 2014.
        if ~isempty(Propose3('get','p12')),
            x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p1z=double(real(squeeze(pData.p12(round(end/2),round(end/2),:))));
            p1z=p1z.*1e-6;
            %p1z=sign(p1z).*(20*log10(abs(p1z)));
            plot(t,p1z,'y');ylim([(min(p1z(:))-0.1),max(p1z(:))+0.1]);
            xlim([-5 5]);
            ylabel('Sound pressure level [MPa]','interpreter','latex','FontSize',11);
            title(['Fundamental z= ',num2str(1e3*pData.z21),' mm'],'interpreter','latex','FontSize',11);
            xlabel('Retarded time [$\mu s$]','interpreter','latex','FontSize',11);
            
        end;
        if ~isempty(Propose3('get','p22')),
            x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p2z=double(real(squeeze(pData.p22(round(end/2),round(end/2),:))));
            p2z=p2z.*1e-6;
            %p2z=sign(p2z).*(20*log10(abs(p2z)));
            plot(t,p2z);ylim([(min(p2z(:))-0.1),max(p2z(:))+0.1]);
            xlim([-5 5])
            title(['Second harmonic. z= ',num2str(1e3*pData.z22),' mm'],'interpreter','latex','FontSize',11);
            ylabel('Sound pressure level (p) [MPa]','interpreter','latex','FontSize',11);
            xlabel('z-axis (retarded time) [$\mu s$]','interpreter','latex','FontSize',11);
            
        end;
    case {'dBsignal'} %Added by HS - 2014.
        if ~isempty(Propose3('get','p12')),
            if ~isempty(Propose3('get','p22')),
                subplot(2,1,1);
            end
            x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p1z=double(real(squeeze(pData.p12(round(end/2),round(end/2),:))));
            p1z=TodB(p1z);
            plot(t,p1z,'g');
            
            ylim([-dynRange,dynRange]);xlim([-5 5]);
            ylabel('Sound pressure level [MPa]','interpreter','latex');
            title(['Fundamental z= ',num2str(1e3*pData.z21),' mm'],'interpreter','latex');
            xlabel('z-axis (retarded time) [$\mu s$]','interpreter','latex'); 
        end;
        if ~isempty(Propose3('get','p22')),
            subplot(2,1,2);
            x=1e3*pData.xax;y=1e3*pData.yax;z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p2z=double(real(squeeze(pData.p22(round(end/2),round(end/2),:))));
            p2z=TodB(p2z);
            plot(t,p2z);
            ylim([-dynRange,dynRange]);xlim([-5 5]);
            title(['Second harmonic. z= ',num2str(1e3*pData.z22),' mm'],'interpreter','latex');
            ylabel('Sound pressure level [dB]','interpreter','latex');
            xlabel('z-axis (retarded time) [$\mu s$]','interpreter','latex');
            
        end
    case {'fundamentalSignal'} %Added by HS - 2014.
        if ~isempty(Propose3('get','p12')),
            z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p1z=double(real(squeeze(pData.p12(round(end/2),round(end/2),:))));
            %p1z=TodB(p1z);
            signal=p1z./max(p1z(:));
            plot(t,signal,'r');
            
            ylim([-1.1 1.1]);
            xlim([-5 5])
            %ylim([-dynRange,dynRange]);xlim([-5 5]);
            ylabel('Pressure [MPa]','interpreter','latex');
            title(['Fundamental z= ',num2str(1e3*pData.z21),' mm'],'interpreter','latex');
            xlabel('z-axis (retarded time) [$\mu s$]','interpreter','latex');
            
        end;
    case {'harmonicSignal'} %Added by HS - 2014.
        if ~isempty(Propose3('get','p22')),
            z=1e3*pData.zax;
            t=(z./C).*1e3; %to get micro seconds as the scale
            p2z=double(real(squeeze(pData.p22(round(end/2),round(end/2),:))));
            %p2z=TodB(p2z);
            signal=p2z./max(p2z(:));
            plot(t,signal,'r');
            ylim([-1.1 1.1]);
            xlim([-5 5])
            %ylim([-dynRange,dynRange]);xlim([-5 5]);
            title(['Second harmonic. z= ',num2str(1e3*pData.z22),' mm'],'interpreter','latex');
            ylabel('Pressure [MPa]','interpreter','latex');
            xlabel('z-axis (retarded time) [$\mu s$]','interpreter','latex');
        end;
        
    case {'aperture'}
        subplot(1,2,1);imagesc(pData.xax,pData.yax,pData.Atx');axis('image');title('Tx aperture');grid;
        subplot(1,2,2);imagesc(pData.xax,pData.yax,pData.Arx');axis('image');title('Rx aperture');grid;
        
    otherwise, display('Unknown display option in Propose');
        
end;


%%%%%%%%%%%%%%%%%%    end of plotResults   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

function waitbar_sh(Noperations)

persistent wh N done  count;
if nargin,
    N=Noperations;
    wh=waitbar(0,'Calculate 2. harm field. ');
    count=0;
    tic;
else
    if ~ishandle(wh),disp('Missing waitbar');return;end;
    count=count+1;
    done=count/N;
    timeused=toc;
    timeleft=toc/done*(1-done);

    minutes=fix(timeused/60);
    seconds=round(timeused)-60*minutes;
    if minutes>0, timeusedStr=[num2str(minutes),' min.',num2str(seconds),' sec.'];
    else timeusedStr=[num2str(round(timeused)),' sec.'];end;

    minutes=fix(timeleft/60);
    seconds=round(timeleft)-60*minutes;
    if minutes>0, timeleftStr=[num2str(minutes),' min.',num2str(seconds),' sec.'];
    else timeleftStr=[num2str(round(timeleft)),' sec.'];end;
    
    waitbar(done,wh,['Time used/left: ',timeusedStr,' / ',timeleftStr]);
    if count==N,
        close(wh);
        disp(['Second harmonic wave simulation time ',timeusedStr]);
    end;
end;  

%%%%%%%%%%%%%%%%%%  end of waitbar_sh %%%%%%%%%%%%%%%%%%%%%%%%%%%%
function returnMatrix=TodB(inputMatrix,signFlag)
%%Converting values of matrix to Db values 
% signFlag (optional): 1 sets sign of Matrix cells to be preserved, 0 sets highest
% positive value in matrix to 0dB (experimental option! - not verified). Default is 1. 2014: Haakon Seljaasen.
if nargin<2;       signFlag=1   ;end

global pSettings;
s=pSettings;
dynRange=s.dynRange;
%normalize to 1
MAX=max(inputMatrix(:));
inputMatrix=inputMatrix./MAX;
%Make dB scale
if signFlag,
    returnMatrix=sign(inputMatrix).*max((20*log10(abs(inputMatrix))+dynRange),0);
else
    %Setting highest value to zero.
    inputMatrix=inputMatrix-1;
    returnMatrix=-(20*log10(abs(inputMatrix)));
end    




function symmetry=symmetryChoice
global pSettings;
symmetry=pSettings.useSymmetry;

function c=speedOfSound
global pSettings;
c=pSettings.c;

function [ro,kappa,betaN]=acousticalProperties
global pSettings;
kappa=pSettings.kappa;
ro=pSettings.ro;
betaN=pSettings.betaN;


function res=testPlot
% set res=1 to enable testplot during simulation
res=0;

function s=calcformat
%s='single'; 
s='double';

function f0=xmitfreq
global pSettings;
f0=pSettings.f0tx;